# -*- coding: utf-8 -*-

from model.network.Node import Node
from model.network.Edge import Edge
from model.network.Graph import Graph
from view.ASubWindow import ASubWindow
from view.network.GraphWidget import GraphWidget

from PyQt5.QtCore import (
    Qt, QRect, QVariant, QAbstractTableModel, pyqtSlot, pyqtSignal,
)

from PyQt5.QtWidgets import (
    QTableView, QItemDelegate, QComboBox, QLineEdit, QHBoxLayout, QSlider,
    QPushButton,
)

class LineEditDelegate(QItemDelegate):
    on_focus_out = pyqtSignal(object)

    def __init__(self, *args, **kwargs):
        super(LineEditDelegate, self).__init__(*args, **kwargs)
        self.line_edit = None

    def createEditor(self, parent, option, index):
        self.line_edit = QLineEdit(parent=parent)
        self.line_edit.destroyed.connect(lambda: self._line_edit_left(index=index))
        return self.line_edit

    def _line_edit_left(self, index):
        self.on_focus_out.emit(index)
        return

class ComboBoxDelegate(QItemDelegate):
    def __init__(self, graph=None, parent=None):
        super(ComboBoxDelegate, self).__init__(parent)
        self.graph = graph

    def createEditor(self, parent, option, index):
        self.editor = QComboBox(parent)
        self.editor.addItems(self.graph.nodes_names())
        return self.editor

    def setEditorData(self, editor, index):
        value = index.data(Qt.DisplayRole)
        self.editor.currentTextChanged.connect(self.currentItemChanged)

    def setModelData(self, editor, model, index):
        text = str(editor.currentText())
        model.setData(index, text)
        editor.close()
        editor.deleteLater()

    def updateEditorGeometry(self, editor, option, index):
        r = QRect(option.rect)
        if self.editor.windowFlags() & Qt.Popup and editor.parent() is not None:
            r.setTopLeft(self.editor.parent().mapToGlobal(r.topLeft()))
        editor.setGeometry(r)

    @pyqtSlot()
    def currentItemChanged(self):
        self.commitData.emit(self.sender())


class TableModel(QAbstractTableModel):
    def __init__(self, headers=[], graph=None, rows_type="nodes"):
        super(QAbstractTableModel, self).__init__()
        self.headers = headers
        self.graph = graph
        self._type = rows_type

        if self._type == "nodes":
            self.rows = graph.nodes()
        elif self._type == "edges":
            self.rows = graph.edges()

    def flags(self, index):
        return Qt.ItemIsEditable | Qt.ItemIsEnabled | Qt.ItemIsSelectable

    def rowCount(self, parent):
        return len(self.rows)

    def columnCount(self, parent):
        return len(self.headers)

    def data(self, index, role):
        if role != Qt.ItemDataRole.DisplayRole:
            return QVariant()

        return self.rows[index.row()][self.headers[index.column()]]

    def headerData(self, section, orientation, role):
        if role == Qt.ItemDataRole.DisplayRole and orientation == Qt.Orientation.Horizontal:
            return self.headers[section].capitalize()

        if role == Qt.ItemDataRole.DisplayRole and orientation == Qt.Orientation.Vertical:
            return section

        return QVariant()

    @pyqtSlot()
    def setData(self, index, value, role=Qt.EditRole):
        if index.isValid():
            if role == Qt.EditRole:
                if (self.headers[index.column()] == "node1" or
                    self.headers[index.column()] == "node2"):
                    node = self.graph.node(value)
                    self.rows[index.row()][self.headers[index.column()]] = node
                else:
                    self.rows[index.row()][self.headers[index.column()]] = value

                self.dataChanged.emit(index, index, [Qt.DisplayRole])
                self.layoutChanged.emit()
                return True

            self.dataChanged.emit(index, index)
        else:
            return False

    def update(self):
        if self._type == "nodes":
            self.rows = self.graph.nodes()
        elif self._type == "edges":
            self.rows = self.graph.edges()

        self.layoutChanged.emit()

class NetworkWindow(ASubWindow):
    def __init__(self, title="Network", parent=None):
        super(NetworkWindow, self).__init__(name=title, ui="Network", parent=parent)
        self.ui.setWindowTitle(title)

        self.graph = Graph()

        # Graph Widget

        self.graph_widget = GraphWidget(self.graph, parent=self)
        self.graph_layout = self.find(QHBoxLayout, "horizontalLayout_graph")
        self.graph_layout.addWidget(self.graph_widget)
        # self.zoom_slider = QSlider(Qt.Orientation.Vertical, parent=self)
        # self.zoom_slider.setMinimum(0)
        # self.zoom_slider.setMaximum(99)
        # self.zoom_slider.setValue(50)
        # self.graph_layout.addWidget(self.zoom_slider)
        # self.zoom_slider.valueChanged.connect(self.graph_widget.scaleViewSlider)

        # Nodes table

        self.nodes_model = TableModel(
            headers = ["name", "id", "pos"],
            graph = self.graph,
            rows_type="nodes",
        )
        self.delegate_line = LineEditDelegate(parent=self)
        table = self.find(QTableView, "tableView_nodes")
        table.setModel(self.nodes_model)
        table.setItemDelegate(self.delegate_line)
        #table.resizeColumnsToContents()

        # Edges table

        self.reachs_model = TableModel(
            headers = ["name", "node1", "node2"],
            graph = self.graph,
            rows_type="edges"
        )
        self.delegate_combobox = ComboBoxDelegate(
            graph = self.graph,
            parent = self,
        )
        table = self.find(QTableView, "tableView_reachs")
        table.setModel(self.reachs_model)
        table.setItemDelegateForColumn(1, self.delegate_combobox)
        table.setItemDelegateForColumn(2, self.delegate_combobox)
        #table.resizeColumnsToContents()

        # Connection

        self.nodes_model.dataChanged.connect(self.reachs_model.update)
        self.nodes_model.dataChanged.connect(self.graph_widget.rename_nodes)
        self.reachs_model.dataChanged.connect(self.nodes_model.update)
        self.graph_widget.changeEdge.connect(self.reachs_model.update)
        self.graph_widget.changeNode.connect(self.nodes_model.update)

        self.find(QPushButton, "pushButton_add").clicked.connect(
            self.clicked_add
        )
        self.find(QPushButton, "pushButton_del").clicked.connect(
            self.clicked_del
        )

    def clicked_add(self):
        if self.get_push_button_checkable("pushButton_add"):
            self.set_push_button_checkable("pushButton_del", False)
            self.graph_widget.state("add")
        else:
            self.graph_widget.state("move")

    def clicked_del(self):
        if self.get_push_button_checkable("pushButton_del"):
            self.set_push_button_checkable("pushButton_add", False)
            self.graph_widget.state("del")
        else:
            self.graph_widget.state("move")

    def keyPressEvent(self, event):
        key = event.key()

        if key == Qt.Key_Escape:
            self.graph_widget.reset_selection
